/**
 * @file charge.c
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Chrage library
 * @version 0.1
 * @date 2022-04-13
 *
 * @copyright Copyright (c) 2021
 *
 */
#include "charge.h"

bool pos_contrain_in_universe(vec2 pos) {
    return (pos.x <= UNIV_MAX_X && pos.x >= UNIV_MIN_X && pos.y <= UNIV_MAX_Y &&
            pos.y >= UNIV_MIN_Y);
}

vec2 position_to_coordinates(int width, int height, vec2 pos) {
    return vec2_create((int)round(width * (pos.x - UNIV_MIN_X)),
                       (int)round(height * (pos.y - UNIV_MIN_Y)));
}

bool pos_not_too_close(vec2 p, charge_t *c, int num_charges) {
    for (int i = 0; i < num_charges; i++) {
        double norm = vec2_norm(vec2_sub(p, c[i].pos));
        if (norm < EPS)
            return false;
    }
    return true;
}

bool compute_e(charge_t c, vec2 p, double eps, vec2 *e) {
    vec2 qP = vec2_sub(p, c.pos);
    double norm = vec2_norm(qP);
    double E = K * (c.q * 10e-6) / pow(norm, 2); // E = k * (Q / r^2)
    *e = vec2_mul(vec2_div(qP, norm), E);        // e = E * qP / ||qP||
    return (norm > eps);
}

bool compute_total_normalized_e(charge_t *charges, int num_charges, vec2 p,
                                double eps, vec2 *e) {
    vec2 sum;
    vec2 tmp_e;

    for (int i = 0; i < num_charges; i++) {
        if (!compute_e(charges[i], p, eps, &tmp_e))
            return false;
        sum = vec2_add(sum, tmp_e);
    }

    *e = sum;
    return true;
}

bool compute_p_next(charge_t *charges, int num_charges, double eps,
                    double delta, vec2 *p, bool opposed) {
    vec2 v;
    bool can_draw =
        compute_total_normalized_e(charges, num_charges, *p, eps, &v);

    v = vec2_div(vec2_mul(v, delta), vec2_norm(v));

    // P ± deltax * E / ||E||
    *p = vec2_add(*p, vec2_mul(v, opposed ? -1 : 1));

    return can_draw;
}

charge_t charge_create(double q, vec2 pos) {
    charge_t c = {.q = q, .pos = pos};
    return c;
}

double random_charge_q(bool positive) {
    return (rand() % 100) * (positive ? 1 : -1);
}

vec2 electristatic_force(charge_t a, charge_t b) {
    vec2 rab = vec2_sub(b.pos, a.pos);
    vec2 res = vec2_mul(
        rab, -K * (((a.q * 10e-6) * (b.q * 10e-6)) / pow(vec2_norm(rab), 2)));
    return res;
}

vec2 resulting_electrostatic_force(charge_t *charges, int num_charges,
                                   charge_t a) {
    vec2 resulting = vec2_create_zero();

    for (int i = 0; i < num_charges; i++) {
        // Very unlickly to have two charges with the exact same charge
        if (charges[i].q == a.q)
            continue;

        vec2 ef = electristatic_force(a, charges[i]);
        resulting = vec2_add(resulting, ef);
    }

    return resulting;
}

vec2 compute_acceleration(charge_t c, charge_t *charges, int num_charges) {
    return vec2_mul(resulting_electrostatic_force(charges, num_charges, c),
                    1.0 / CHARGE_MASS);
}

vec2 compute_first_pos(charge_t c, charge_t *charges, int num_charges) {
    vec2 a = compute_acceleration(c, charges, num_charges);
    vec2 pos = vec2_add(c.pos, vec2_mul(a, pow(DELTA_T, 2) / 2));
    return pos;
}

vec2 compute_next_pos(charge_t c, charge_t *charges, int num_charges) {
    vec2 a = compute_acceleration(c, charges, num_charges);
    vec2 pos = vec2_add(vec2_sub(vec2_mul(c.pos, 2), c.prev_pos),
                        vec2_mul(a, pow(DELTA_T, 2)));
    return pos;
}

void update_simulation(charge_t *charges, int num_charges) {
    for (int i = 0; i < num_charges; i++) {
        vec2 tmp = charges[i].pos;
        charges[i].pos = compute_next_pos(charges[i], charges, num_charges);
        charges[i].prev_pos = tmp;
    }
}