/**
 * @file charge.h
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Charge file header
 * @version 0.1
 * @date 2022-04-13
 *
 * @copyright Copyright (c) 2021
 *
 */
#ifndef _CHARGE_H_
#define _CHARGE_H_

#include "../utilities/utils.h"
#include "../vec2/vec2.h"
#include <math.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#define K 8.988e9
#define ELEMENTARY_CHARGE 1.602176634e-19
#define CHARGE_RADIUS 12
#define EPS 0.015
#define DELTA_T 10e-4
#define CHARGE_MASS 1500
#define UNIV_MIN_X 0
#define UNIV_MAX_X 1
#define UNIV_MIN_Y 0
#define UNIV_MAX_Y 1

typedef struct _charge_t {
    double q;
    vec2 pos;
    vec2 prev_pos;
} charge_t;

/**
 * @brief Check whether a pos is in the universe or not
 *
 * @param pos
 * @return true
 * @return false
 */
bool pos_contrain_in_universe(vec2 pos);

/**
 * @brief Convert a position ([0, 1]x[0, 1]) to a screen position
 *
 * @param width
 * @param height
 * @param pos
 * @return vec2
 */
vec2 position_to_coordinates(int width, int height, vec2 pos);

/**
 * @brief Check whether a position is too close to a charge or not
 *
 * @param p
 * @param c
 * @param num_charges
 * @return true
 * @return false
 */
bool pos_not_too_close(vec2 p, charge_t *c, int num_charges);

/**
 * @brief Compute E*qP/norm(qP)
 *
 * @param c Parent charge
 * @param p Point on which to compute e
 * @param eps Minimal gap between field and charge
 * @param e Electric field
 * @return bool false norm(qP) < eps
 */
bool compute_e(charge_t c, vec2 p, double eps, vec2 *e);

/**
 * @brief Compute the normilized sum of Ei*qiP/norm(qiP)
 *
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param p Current computed point
 * @param eps Minimal gap between field and charge
 * @param e Electric field
 * @return bool false for some qiP, norm(qiP) < eps
 */
bool compute_total_normalized_e(charge_t *charges, int num_charges, vec2 p,
                                double eps, vec2 *e);

/**
 * @brief Compute the next particle
 *
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param eps Minimal gap between field and charge
 * @param delta Distance between two points in the field line
 * @param p Current point
 * @param opposed Is the next point towards charge
 * @return true
 * @return false
 */
bool compute_p_next(charge_t *charges, int num_charges, double eps,
                    double delta, vec2 *p, bool opposed);

/**
 * @brief Create a charge
 *
 * @param q Electric charge
 * @param pos Position
 * @return charge_t
 */
charge_t charge_create(double q, vec2 pos);

/**
 * @brief Return a random charge q
 *
 * @param positive
 * @return double
 */
double random_charge_q(bool positive);

/**
 * @brief Compute the electrostatic force between two charges
 *
 * @param a
 * @param b
 * @return vec2
 */
vec2 electristatic_force(charge_t a, charge_t b);

/**
 * @brief Compute the resulting electrostatic force
 *
 * @param charges
 * @param num_charges
 * @param a
 * @return vec2
 */
vec2 resulting_electrostatic_force(charge_t *charges, int num_charges,
                                   charge_t a);

/**
 * @brief Compute the first pos for the dynamic simulation
 *
 * @param c
 * @param charges
 * @param num_charges
 * @return vec2
 */
vec2 compute_first_pos(charge_t c, charge_t *charges, int num_charges);

/**
 * @brief Update the whole simulation
 *
 * @param charges
 * @param num_charges
 */
void update_simulation(charge_t *charges, int num_charges);

#endif