/**
 * @file draw.c
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Drawing library
 * @version 0.1
 * @date 2022-04-13
 *
 * @copyright Copyright (c) 2021
 *
 */

#include "draw.h"

//=====================
//      PRIVATE
//=====================
/**
 * @brief Draw a circle
 *
 * @param center
 * @param rad
 */
void draw_circle(vec2 center, int rad) {
    int i;
    int lineAmount = 100; //# of triangles used to draw circle

    // GLfloat radius = 0.8f; //radius
    GLfloat twicePi = 2.0f * 3.141592;

    glBegin(GL_POLYGON);
    for (i = 0; i <= lineAmount; i++) {
        glVertex2f(center.x + (rad * cos(i * twicePi / lineAmount)),
                   center.y + (rad * sin(i * twicePi / lineAmount)));
    }
    glEnd();
}

/**
 * @brief Draw a line
 *
 * @param start
 * @param end
 */
void draw_line(vec2 start, vec2 end) {
    glBegin(GL_LINES);
    glVertex2f(start.x, start.y);
    glVertex2f(end.x, end.y);
    glEnd();
}

void get_heatmap_color(double value, double *red, double *green, double *blue) {
    double aR = 1.0;
    double aG = 1.0;
    double aB = 1.0; // RGB for our 1st color
    double bR = 0.0;
    double bG = 0.0;
    double bB = 0.47; // RGB for our 2nd color

    *red = (bR - aR) * value + aR * 1.5;   // Evaluated as -255*value + 255.
    *green = (bG - aG) * value + aG * 1.5; // Evaluates as 0.
    *blue = (bB - aB) * value + aB * 1.5;  // Evaluates as 255*value + 0.
}

//=====================
//      PUBLIC
//=====================
void draw_textbox(vec2 pos, char *text) {
    int text_len = strlen(text);
    int textbox_width = CHAR_WIDTH * text_len + 8;
    int textbox_height = CHAR_HEIGHT + 4;

    // Background rect
    glBegin(GL_POLYGON);
    glColor3f(0, 0, 0);
    glVertex2d(pos.x, pos.y);
    glVertex2d(pos.x + textbox_width, pos.y);
    glVertex2d(pos.x + textbox_width, pos.y + textbox_height);
    glVertex2d(pos.x, pos.y + textbox_height);
    glEnd();

    glBegin(GL_LINE_LOOP);
    glColor3f(1, 1, 1);
    glVertex2d(pos.x, pos.y);
    glVertex2d(pos.x + textbox_width, pos.y);
    glVertex2d(pos.x + textbox_width, pos.y + textbox_height);
    glVertex2d(pos.x, pos.y + textbox_height);
    glEnd();

    // Text
    glColor3f(1, 1, 1);
    glRasterPos2i(pos.x + 4, pos.y + CHAR_HEIGHT);
    glutBitmapString(GLUT_BITMAP_8_BY_13, (unsigned char *)text);
}

void draw_field_line_segment(charge_t *charges, int num_charges, double dx,
                             int w, int h, vec2 p, vec2 next, bool side) {
    while (pos_contrain_in_universe(next) &&
           pos_not_too_close(next, charges, num_charges)) {
        vec2 old_coord = position_to_coordinates(w, h, p);
        vec2 p_coord = position_to_coordinates(w, h, next);

        draw_line((vec2){p_coord.x, p_coord.y},
                  (vec2){old_coord.x, old_coord.y});

        p = next; // Save current point
        compute_p_next(charges, num_charges, EPS, dx, &next, side);
    }
}

void draw_field_line(charge_t *charges, int num_charges, double dx, vec2 pos0,
                     int w, int h) {
    vec2 pos1 = pos0;

    draw_field_line_segment(charges, num_charges, dx, w, h, pos0, pos1, true);
    draw_field_line_segment(charges, num_charges, dx, w, h, pos0, pos1, false);
}

void draw_heatmap(charge_t *charges, int num_charges, int w, int h) {
    vec2 *intensities = malloc(sizeof(vec2) * (w * h));
    double min = DBL_MAX, max = 0;

    int i = 0;
    for (double y = 0; y < 1.0 - 1.0 / h; y += 1.0 / h) {
        for (double x = 0; x < 1.0 - 1.0 / w; x += 1.0 / w) {
            vec2 p = vec2_create(x, y);
            if (!compute_total_normalized_e(charges, num_charges, p, EPS,
                                            &intensities[i]))
                intensities[i] = vec2_create(0, 0);

            double in = vec2_norm(intensities[i]);
            if (in < min && in > 1)
                min = in;
            if (in > max)
                max = in;

            i++;
        }
    }

    for (int i = 0; i < h * w; i++) {
        int x = i % w;
        int y = i / w;
        double r, g, b;
        double a = (vec2_norm(intensities[i]) / max) * 40.0;

        get_heatmap_color(a, &r, &g, &b);
        glBegin(GL_POINTS);
        glColor3f(r, g, b);
        glVertex2d(x, y);
        glEnd();
    }

    free(intensities);
}

void draw_vector_field(charge_t *charges, int num_charges, vec2 *points,
                       int nb_points, int w, int h) {
    vec2 vs[nb_points];
    for (int i = 0; i < nb_points; i++) {
        if (!compute_total_normalized_e(charges, num_charges, points[i], EPS,
                                        &vs[i]))
            vs[i] = vec2_create(0, 0);

        vs[i] = vec2_normalize(vs[i]);
    }

    for (int i = 0; i < nb_points; i++) {
        vec2 a = position_to_coordinates(w, h, points[i]);

        glColor3f(0.18, 0.46, 0.18);
        draw_circle(a, 3);

        glLineWidth(2);
        glBegin(GL_LINES);
        glVertex2d(a.x, a.y);
        a = vec2_add(a, vec2_mul(vs[i], 20));
        glVertex2d(a.x, a.y);
        glEnd();
        glLineWidth(1);
    }
}

void draw_charges(charge_t *charges, int num_charges, int w, int h) {
    int chargePadding = CHARGE_RADIUS / 2;

    // Draw charges
    for (int i = 0; i < num_charges; i++) {
        vec2 chCoord = position_to_coordinates(w, h, charges[i].pos);

        // Draw sign according to charge value
        if (charges[i].q < 0) {
            glColor3f(0.54, 0.82, 0.90);
            draw_line((vec2){chCoord.x - chargePadding, chCoord.y},
                      (vec2){chCoord.x + chargePadding, chCoord.y});
        } else {
            glColor3f(1, 0.4, 0.38);
            draw_line((vec2){chCoord.x, chCoord.y - chargePadding},
                      (vec2){chCoord.x, chCoord.y + chargePadding});
            draw_line((vec2){chCoord.x - chargePadding, chCoord.y},
                      (vec2){chCoord.x + chargePadding, chCoord.y});
        }

        draw_circle(chCoord, CHARGE_RADIUS);

        char charge_str[8];
        snprintf(charge_str, 8, "%.0f", charges[i].q);
        vec2 text_pos = vec2_create(chCoord.x, chCoord.y + CHARGE_RADIUS + 4);
        draw_textbox(text_pos, charge_str);
    }
}