/**
 * @file main.c
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Main file for the electric field exercice
 * @version 0.1
 * @date 2022-04-13
 *
 * @copyright Copyright (c) 2021
 *
 */
#include "charge/charge.h"
#include "draw/draw.h"
#include "utilities/utils.h"
#include <GL/freeglut.h>
#include <GL/glut.h>
#include <math.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// https://stackoverflow.com/questions/3417837/what-is-the-best-way-to-suppress-a-unused-variable-x-warning
#ifdef UNUSED
#elif defined(__GNUC__)
#define UNUSED(x) UNUSED_##x __attribute__((unused))
#elif defined(__LCLINT__)
#define UNUSED(x) /*@unused@*/ x
#else
#define UNUSED(x) x
#endif

#define ONE_SECOND_IN_MILLISECONDS 1000
#define REFRESH_RATE 60

#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 800
#define NB_CHARGES 6
#define GRID_SPACING 15
#define NB_POINTS                                                              \
    ((WINDOW_WIDTH / GRID_SPACING) * (WINDOW_WIDTH / GRID_SPACING))
#define MAXIMUM_CHARGES 30 // Can be replace by a dynamic array
#define DELTA (1 / sqrt(pow(WINDOW_WIDTH, 2) + pow(WINDOW_HEIGHT, 2)))

int charge_count = NB_CHARGES;
vec2 starting_points[NB_POINTS];
vec2 default_charges_pos[MAXIMUM_CHARGES];
double default_charges[MAXIMUM_CHARGES];
charge_t *charges;

bool toggle_lines_view = true;
bool toggle_vector_view = false;
bool toggle_heatmap_view = false;
bool animate = false;

void update() { update_simulation(charges, charge_count); };

void update_timer() {
    if (animate)
        update();
    glutTimerFunc(ONE_SECOND_IN_MILLISECONDS / REFRESH_RATE, update_timer, 0);
}

void draw_timer() {
    glutPostRedisplay();
    glutTimerFunc(ONE_SECOND_IN_MILLISECONDS / REFRESH_RATE, draw_timer, 0);
}

/**
 * @brief Generate a grid of points
 *
 * @param points
 */
void generate_points(vec2 points[]) {
    int i = 0;

    for (int y = GRID_SPACING; y < WINDOW_HEIGHT; y += GRID_SPACING) {
        for (int x = GRID_SPACING; x < WINDOW_WIDTH; x += GRID_SPACING) {
            points[i] = vec2_create(x / (double)WINDOW_WIDTH,
                                    y / (double)WINDOW_HEIGHT);
            i++;
        }
    }
}

/**
 * @brief Display function of Glut
 *
 */
void display() {
    glClearColor(1, 1, 1, 1);
    glClear(GL_COLOR_BUFFER_BIT);

    if (toggle_heatmap_view)
        draw_heatmap(charges, charge_count, WINDOW_WIDTH, WINDOW_HEIGHT);

    if (toggle_lines_view) {
        for (int i = 0; i < NB_POINTS; i += 10) {
            glColor3f(0.2, 0.2, 0.2);
            draw_field_line(charges, charge_count, DELTA, starting_points[i],
                            WINDOW_WIDTH, WINDOW_HEIGHT);
        }
    }

    if (toggle_vector_view) {
        draw_vector_field(charges, charge_count, starting_points, NB_POINTS,
                          WINDOW_WIDTH, WINDOW_HEIGHT);
    }

    draw_charges(charges, charge_count, WINDOW_WIDTH, WINDOW_HEIGHT);

    glutSwapBuffers();
}

/**
 * @brief Event that retrieves actions performed with the mouse.
 *
 * @param button
 * @param state
 * @param x
 * @param y
 */
void handle_mouse_input(int button, int state, int x, int y) {
    if (state == GLUT_UP)
        return;

    // Define new charges attributes
    if (button == GLUT_LEFT_BUTTON || button == GLUT_RIGHT_BUTTON) {
        charge_count += 1;
        default_charges_pos[charge_count - 1] =
            vec2_create((double)x / WINDOW_WIDTH, (double)y / WINDOW_HEIGHT);
    }

    if (button == GLUT_LEFT_BUTTON)
        default_charges[charge_count - 1] = random_charge_q(true);

    if (button == GLUT_RIGHT_BUTTON)
        default_charges[charge_count - 1] = random_charge_q(false);

    charges[charge_count - 1] =
        charge_create(default_charges[charge_count - 1],
                      default_charges_pos[charge_count - 1]);
    charges[charge_count - 1].prev_pos = charges[charge_count - 1].pos;
    charges[charge_count - 1].pos =
        compute_first_pos(charges[charge_count - 1], charges, charge_count);

    glutPostRedisplay();
}

/**
 * @brief Toggle some views
 *
 * @param key
 * @param x
 * @param y
 */
void keyboard(unsigned char key, int UNUSED(x), int UNUSED(y)) {
    switch (key) {
    case 'l':
        toggle_lines_view = !toggle_lines_view;
        break;
    case 'v':
        toggle_vector_view = !toggle_vector_view;
        break;
    case 'h':
        toggle_heatmap_view = !toggle_heatmap_view;
        break;
    case ' ':
        animate = !animate;
        break;
    }

    glutPostRedisplay();
}

/**
 * @brief Main function
 *
 * @param argc
 * @param argv
 * @return int
 */
int main(int argc, char **argv) {
    srand(time(NULL));

    charges = malloc(sizeof(charge_t) * charge_count *
                     10); // Large list for further charges

    // Generate all default charges
    for (int i = 0; i < charge_count; i++) {
        default_charges[i] = random_charge_q((bool)(rand() % 2 == 0));
        default_charges_pos[i] = vec2_create(rand_one(), rand_one());
    }

    // Init charges
    for (int i = 0; i < charge_count; i++) {
        charges[i] = charge_create(default_charges[i], default_charges_pos[i]);
        charges[i].prev_pos = charges[i].pos;
        charges[i].pos = compute_first_pos(charges[i], charges, charge_count);
    }

    // Generate all field lines starting points
    generate_points(starting_points);

    // Window initialization
    glutInit(&argc, argv);
    glutSetOption(GLUT_MULTISAMPLE, 16);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH |
                        GLUT_MULTISAMPLE);
    glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
    int window = glutCreateWindow("GL RGB Triangle");

    glClearColor(0, 0, 0, 1);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, WINDOW_WIDTH, WINDOW_HEIGHT, 0);

    glutDisplayFunc(display);
    glutMouseFunc(handle_mouse_input);
    glutKeyboardFunc(keyboard);

    glutTimerFunc(ONE_SECOND_IN_MILLISECONDS / REFRESH_RATE, update_timer, 0);
    glutTimerFunc(ONE_SECOND_IN_MILLISECONDS / REFRESH_RATE, draw_timer, 0);

    glutMainLoop();

    // Object destroy
    glutDestroyWindow(window);
    free(charges);
    return EXIT_SUCCESS;
}